#!/usr/bin/env python3
# 
# pycman - A Python implementation of Pacman
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
pycman configuration handling

This module handles pacman.conf files as well as pycman options that
are common to all action modes.
"""

import argparse
import configparser

import pyalpm

def read_config(path):
	parser = configparser.ConfigParser(
			allow_no_value = True,
			delimiters = ['='],
			comment_prefixes = ['#'],
			empty_lines_in_values = False)
	parser.read(path)
	return parser

def make_parser(*args, **kwargs):
	parser = argparse.ArgumentParser(*args, **kwargs)
	common = parser.add_argument_group('Common options')
	common.add_argument('-b', '--dbpath', metavar = '<path>',
			action = 'store', dest = 'dbpath', type = str,
			help = 'set an alternate database location')
	common.add_argument('-r', '--root', metavar = '<path>',
			action = 'store', dest = 'root', type = str,
			help = 'set an alternate installation root')
	common.add_argument('-v', '--verbose',
			action = 'store_true', dest = 'verbose', default = False,
			help = 'be verbose')
	common.add_argument('--arch', metavar = '<arch>',
			action = 'store', dest = 'arch', type = str,
			help = 'set an alternate architecture')
	common.add_argument('--config', metavar = '<file>',
			action = 'store', dest = 'config', type = str,
			help = 'set an alternate configuration file')
	common.add_argument('--logfile', metavar = '<file>',
			action = 'store', dest = 'logfile', type = str,
			help = 'set an alternate log file')
	return parser

def init_with_config(options):
	pyalpm.initialize()

	# read config file
	if options.config is not None:
		config_file = options.config
	else:
		config_file = "/etc/pacman.conf"
	config = read_config(config_file)

	try:
		config_options = config["options"]
	except KeyError:
		config_options = {}

	# set libalpm options
	if options.root is not None:
		pyalpm.options.root = options.root
	else:
		pyalpm.options.root = config_options.get("rootdir", "/")

	if options.dbpath is not None:
		pyalpm.options.dbpath = options.dbpath
	else:
		pyalpm.options.dbpath = config_options.get("dbpath", "/var/lib/pacman")

	if options.arch is not None:
		pyalpm.options.arch = options.arch
	else:
		pyalpm.options.arch = config_options.get("architecture", "auto")

	if options.logfile is not None:
		pyalpm.options.logfile = options.logfile
	else:
		pyalpm.options.logfile = config_options.get("logfile", "/var/log/pacman.log")

	# set sync databases
	for repo in config.sections():
		if repo == "options":
			continue
		pyalpm.register_syncdb(repo)

# vim: set ts=4 sw=4 tw=0 noet:



#!/usr/bin/env python3
# 
# pycman-sync - A Python implementation of Pacman
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
A Python implementation of pacman -S

This script displays information about packages available in repositories,
and is also used to install/upgrade/remove them.
"""

import sys

import pyalpm
from pycman import config
from pycman import pkginfo

def show_groups(args):
	"Show groups like pacman -Sg"
	for repo in pyalpm.get_syncdbs():
		if len(args.args) == 0:
			# list all available groups
			[print(name) for name, pkgs in repo.grpcache]
		else:
			# only print chosen groups
			for group in args.args:
				grp = repo.read_grp(group)
				if grp is None:
					continue
				else:
					name, pkgs = grp
				if args.quiet:
					[print(pkg.name) for pkg in pkgs]
				else:
					[print(name, pkg.name) for pkg in pkgs]
	return 0

def show_repo(args):
	"Show repository's list of packages like pacman -Sl"
	repos = pyalpm.get_syncdbs()
	if len(args.args) > 0:
		repo_dict = dict((repo.name, repo) for repo in repos)
		try:
			repos = [repo_dict[name] for name in args.args]
		except KeyError as err:
			print("error: repository '%s' was not found" % err.args)
			return 1

	for repo in repos:
		if args.quiet:
			[print(pkg.name) for pkg in repo.pkgcache]
		else:
			[print(repo.name, pkg.name, pkg.version) for pkg in repo.pkgcache]
	return 0

def show_packages(args):
	"Show information about packages like pacman -Si"
	retcode = 0
	repos = pyalpm.get_syncdbs()
	if len(args.args) == 0:
		for repo in repos:
			for pkg in repo.pkgcache:
				pkginfo.display_pkginfo(pkg, level = args.info, style = 'sync')
	else:
		for pkgname in args.args:
			if '/' in pkgname:
				repo, pkgname = pkgname.split('/', 1)
				pkg = repo.get_pkg(pkgname)
				if pkg is None:
					retcode = 1
					print("error: package '%s' was not found in repository '%s'" % (pkgname, repo.name))
				else:
					pkginfo.display_pkginfo(pkg, level = args.info, style = 'sync')
			else:
				found = False
				for repo in repos:
					pkg = repo.get_pkg(pkgname)
					if pkg is None:
						continue
					found = True
					pkginfo.display_pkginfo(pkg, level = args.info, style = 'sync')
				if not found:
					retcode = 1
					print("error: package '%s' was not found" % pkgname)
	return retcode

def main(rawargs):
	parser = config.make_parser(prog = 'pycman-sync')
	group = parser.add_argument_group("sync options")
	group.add_argument('-i', '--info',
			action = 'count', dest = 'info', default = 0,
			help = 'view package information')
	group.add_argument('-g', '--groups', action = 'store_true', default = False,
			help = 'view list of groups, or all members of a package group')
	group.add_argument('-l', '--list', action = 'store_true', default = False,
			help = 'list the contents of repositories')
	group.add_argument('-q', '--quiet',
			action = 'store_true', dest = 'quiet', default = False,
			help = 'show less information for query and search')
	group.add_argument('args', metavar = 'arg', nargs = '*',
			help = 'arguments (group names for -g, repo names for -l, '
			'package names for -i)')

	args = parser.parse_args(rawargs)
	config.init_with_config(args)

	if args.verbose:
		print("sync " + " ".join(rawargs), file = sys.stderr)

	if args.groups:
		return show_groups(args)
	elif args.info:
		return show_packages(args)
	elif args.list:
		return show_repo(args)

	return 0

if __name__ == "__main__":
	ret = main(sys.argv[1:])
	sys.exit(ret)

# vim: set ts=4 sw=4 tw=0 noet:



#!/usr/bin/env python3
# 
# pycman-query - A Python implementation of Pacman
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
A Python implementation of pacman -Q

This script displays information about installed packages.
"""

import sys

import pyalpm
from pycman import config
from pycman import pkginfo

def display_pkg(pkg, options):
	if options.info > 0:
		pkginfo.display_pkginfo(pkg, level = options.info, style = 'local')
	elif not options.listfiles:
		if options.quiet:
			print(pkg.name)
		else:
			print(pkg.name, pkg.version)

	if options.listfiles:
		if options.quiet:
			[print('/' + f) for f in pkg.files]
		else:
			[print(pkg.name, '/' + f) for f in pkg.files]

def main(rawargs):
	parser = config.make_parser(prog = 'pycman-query')
	group = parser.add_argument_group("Query options")
	group.add_argument('-i', '--info',
			action = 'count', dest = 'info', default = 0,
			help = 'view package information')
	group.add_argument('-l', '--list',
			action = 'store_true', dest = 'listfiles', default = False,
			help = 'list the contents of the queried package')
	group.add_argument('-q', '--quiet',
			action = 'store_true', dest = 'quiet', default = False,
			help = 'show less information for query and search')
	group.add_argument('pkgnames', metavar = 'pkg', nargs = '*',
			help = 'packages to show (show all packages if no arguments)')

	args = parser.parse_args(rawargs)
	config.init_with_config(args)

	if args.verbose:
		print("query " + " ".join(rawargs), file = sys.stderr)

	db = pyalpm.get_localdb()
	retcode = 0

	if len(args.pkgnames) > 0:
		# a list of package names was specified
		for pkgname in args.pkgnames:
			pkg = db.get_pkg(pkgname)
			if pkg is None:
				print('error: package "%s" not found' % pkgname)
				retcode = 1
			else:
				display_pkg(pkg, args)
	else:
		# no package was specified, display all
		for pkg in db.pkgcache:
			display_pkg(pkg, args)

	return retcode

if __name__ == "__main__":
	ret = main(sys.argv[1:])
	sys.exit(ret)

# vim: set ts=4 sw=4 tw=0 noet:



#!/usr/bin/env python3
# 
# pycman.pkginfo - A Python implementation of Pacman
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
Package information formatting

This module defines utility function to format package information
for terminal output.
"""

import time
import textwrap
import pyalpm

ATTRNAME_FORMAT = '%-14s : '
ATTR_INDENT = 17 * ' '

def format_attr(attrname, value, format = None):
	if isinstance(value, list):
		if len(value) == 0:
			valuestring = 'None'
		else:
			valuestring = '  '.join(str(v) for v in value)
	else:
		if format == "time":
			valuestring = time.strftime("%a %d %b %Y %X %Z", time.localtime(value))
		else:
			valuestring = str(value)
	return textwrap.fill(valuestring, width = 80,
			initial_indent = ATTRNAME_FORMAT % attrname,
			subsequent_indent = ATTR_INDENT,
			break_on_hyphens = False,
			break_long_words = False)

def format_attr_oneperline(attrname, value):
	if len(value) == 0:
		value = ['None']
	s = ATTRNAME_FORMAT % attrname
	s += ('\n' + ATTR_INDENT).join(value)
	return s

def display_pkginfo(pkg, level = 1, style = 'local'):
	"""
	Displays pretty-printed package information.

	Args :
	  pkg -- the package to display
	  level -- the level of detail (1 or 2)
	  style -- 'local' or 'sync'
	"""

	if style not in ['local', 'sync']:
		raise ValueError('Invalid style for package info formatting')

	if style == 'sync':
		print(format_attr('Repository', pkg.db.name))
	print(format_attr('Name', pkg.name))
	print(format_attr('Version', pkg.version))
	print(format_attr('URL', pkg.url))
	print(format_attr('Licenses', pkg.licenses))
	print(format_attr('Groups', pkg.groups))
	print(format_attr('Provides', pkg.provides))
	print(format_attr('Depends on', pkg.depends))
	print(format_attr_oneperline('Optional deps', pkg.optdepends))
	if style == 'local' or level == 2:
		print(format_attr('Required by', pkg.compute_requiredby()))
	print(format_attr('Conflicts with', pkg.conflicts))
	print(format_attr('Replaces', pkg.replaces))
	if style == 'sync':
		print(format_attr('Download size', '%.2f K' % (pkg.size / 1024)))
	print(format_attr('Installed size', '%.2f K' % (pkg.isize / 1024)))
	print(format_attr('Packager', pkg.packager))
	print(format_attr('Architecture', pkg.arch))
	print(format_attr('Build Date', pkg.builddate, format = 'time'))

	if style == 'local':
		# local installation information
		print(format_attr('Install Date', pkg.installdate, format = 'time'))
		if pkg.reason == pyalpm.PKG_REASON_EXPLICIT:
			reason = 'Explicitly installed'
		elif pkg.reason == pyalpm.PKG_REASON_DEPEND:
			reason = 'Installed as a dependency for another package'
		else:
			reason = 'N/A'
		print(format_attr('Install Reason', reason))
		print(format_attr('Install Script', 'Yes' if pkg.has_scriptlet else 'No'))
	else:
		print(format_attr('MD5 Sum', pkg.md5sum))

	print(format_attr('Description', pkg.desc))

	if level >= 2 and style == 'local':
		# print backup information
		print('Backup files:')
		if len(pkg.backup) == 0:
			print('(none)')
		else:
			print('\n'.join(["%s %s" % (md5, file) for (file, md5) in pkg.backup]))
	print('')

# vim: set ts=4 sw=4 noet:



# -*- coding: utf-8 -*-
import os
import setuptools

os.putenv('LC_CTYPE', 'en_US.UTF-8')

alpm = setuptools.Extension('pyalpm',
    libraries = ['alpm'],
    sources = [
        'src/pyalpm.c',
        'src/util.c',
        'src/package.c',
        'src/db.c',
        'src/options.c',
        'src/optionsobject.c'
        ])

setuptools.setup(name = 'pyalpm',
      version = '0.2',
      description = 'libalpm bindings for Python 3',
      author = "Rémy Oudompheng",
      author_email = "remy@archlinux.org",
      url = "http://projects.archlinux.org/users/remy/pyalpm.git",
      packages = ["pycman"],
      scripts = ["scripts/pycman"],
      ext_modules = [alpm])
